                 (* +-----------------------------+
                    |  Manipulation de polynmes  |
                    +-----------------------------+ *)

(* addition *)
let prefix ++ p q =
  let lp = vect_length(p)
  and lq = vect_length(q)         in
  let r = make_vect (max lp lq) 0 in
  for i=0 to lp-1 do r.(i) <- p.(i) done;
  for i=0 to lq-1 do r.(i) <- r.(i) + q.(i) done;
  r
;;

(* essai *)
let p = [| 1; 2; 3   |]     (* 1 + 2x + 3x^2 *)
and q = [| 4; 5; 6; 7|]     (* 4 + 5x + 6x^2 + 7x^3 *)
;;

p ++ q;;

(* soustration *)
let prefix -- p q =
  let lp = vect_length(p)
  and lq = vect_length(q) in
  let r = make_vect (max lp lq) 0 in
  for i=0 to lp-1 do r.(i) <- p.(i) done;
  for i=0 to lq-1 do r.(i) <- r.(i) - q.(i) done;
  r
;;

p -- q;;
p -- p;;

(* limination des zros de tte *)
let rduit p =
  let i = ref(vect_length(p)-1) in
  while !i > 0 & p.(!i) = 0 do i := !i-1 done;
  sub_vect p 0 (!i+1)
;;

rduit(p -- p);;

(* multiplication *)
let prefix ** p q =
  let lp = vect_length(p)
  and lq = vect_length(q) in
  let r = make_vect (lp+lq-1) 0 in
  for   i=0 to lp-1 do
    for j=0 to lq-1 do r.(i+j) <- r.(i+j) + p.(i)*q.(j) done
  done;
  r
;;

p ** q;;

(* composition *)
let compose p q =
  let r = ref([|0|]) in
  for i = vect_length(p)-1 downto 0 do r := [|p.(i)|] ++ q**(!r) done;
  rduit(!r)
;;

compose p q;;
compose [|0;0;0;0;0;0;1|] [|1;1|];;

(* affichage *)
#open "format";;
let affiche(p) =
  let lp = vect_length(p)
  and premier = ref(true) in
  for i=0 to lp-1 do
    if p.(i) <> 0 then begin
      if p.(i) < 0 then print_string " - "
      else if not(!premier) then print_string " + ";
      if i=0 or abs(p.(i)) <> 1 then print_int(abs(p.(i)));
      premier := false;
      match i with
	| 0 -> ()
	| 1 -> print_string "x"
	| _ -> print_string "x^"; print_int i
    end
  done;
  if !premier then print_string "0"
;;

install_printer "affiche";;

(* polynmes de Chebychev *)
let chebychev(n) =
  if n = 0 then [|1|]
  else begin
    let p0 = ref([|1|])
    and p1 = ref([|0;1|])
    and deux_x = [|0;2|]  in
    for i=2 to n do
      let p = deux_x ** !p1 -- !p0 in
      p0 := !p1;
      p1 := p
    done;
    !p1
  end
;;

chebychev(0);;
chebychev(1);;
chebychev(2);;
chebychev(3);;
chebychev(10);;


(* division  euclidienne *)
let division a b =

  let la = vect_length(a)
  and lb = vect_length(b) in
  if (b.(lb-1) <> 1) & (b.(lb-1) <> -1) then failwith "division impossible";

  let q  = make_vect (la-lb+1) 0
  and r  = copy_vect a in

  for i=la-lb downto 0 do
    let x = r.(lb+i-1)/b.(lb-1) in
    for j=0 to lb-1 do r.(i+j) <- r.(i+j) - x*b.(j) done;
    q.(i) <- x
  done;

  (q,rduit(r))
;;

division q [|1;1;1|];;

(* polynmes cyclotomiques *)
let cyclotomique(n) =

  let cyclo = make_vect (n+1) [|0|] in
  for i=1 to n do
    if n mod i = 0 then begin
      cyclo.(i) <- make_vect (i+1) 0;
      cyclo.(i).(0) <- -1;
      cyclo.(i).(i) <-  1;
      for j=1 to i-1 do
	if i mod j = 0 then begin
	  let q,_ = division cyclo.(i) cyclo.(j) in
	  cyclo.(i) <- q
	end
      done;
    end
  done;
  cyclo.(n)
;;

let c = cyclotomique(30);;
